\documentclass[a4paper]{article}
% \setbeamertemplate{navigation symbols}{}
\usepackage{url}
\usepackage{eurosym}
\usepackage{textcomp}
\usepackage[utf8]{inputenc}
\usepackage[top=2cm, bottom=2cm, left=2cm, right=2cm]{geometry}
\usepackage{color}
\usepackage{graphicx}
\usepackage{float}
\floatstyle{boxed} 
\restylefloat{figure}
\usepackage{wrapfig}

\newcommand{\mint}[1]{( #1 Min)}

\pagestyle{headings} 
% \markright{x}

\begin{document}

\title{Linux Kurs - Command-Line }
\author{Franz Schäfer}
\date{2019-01-11}
\maketitle

\section{CLI basics  ( 30 min)}

\subsection{Der BASH Prompt}

\texttt{karli$@$meinlaptop:\textasciitilde\$} 

\vspace{0.5cm}

In einer Text-Console wird man/frau üblicherweise mit einem `'Prompt''
begrüst. Der könnte z.b. So aussehen: 


Mit der Eingabetaste bekommt man einen neuen Prompt. Der Prompt lässt sich
konfigurieren. Üblicherweise ist er so eingestellt dass wir sehen unter
welchem User wir arbeiten, auf welchem System wir arbeiten (könnte auch eine
sein, dass am anderen Ende der Welt steht) und das aktuelle Verzeicnis.
ein \$ sagt uns dass wir ``normaler User'' sind. Ein \# ist i.a. für den
root User.

Es gibt verschiedene Shell Programme die uns die Kommandoeingabe
ermöglichen. Default im Linux ist die \textbf{bash}. Alternativen sind
\textbf{tcsh} (default bei Mac OS/X). Beliebt ist auch die \textbf{zsh}.

\subsection{Bewegen im Verzeichnisbau}

Mit den Befehlen \texttt{pwd, cd, df, ls ls -l, ls -ltr} (siehe voriges
Kapital) können wir uns im Verzeichnisbaum bewegen und uns Datein und
Verzeichnisse ansehen.

\subsection{keyboard shortcuts für die BASH}

\begin{description}

   \item[ctrl-p ctrl-n] cursor up/down

   \item[history] der history befehl zeigt die zuletzt eingegen Befehle.

   \item[ctrl-c] Abbruch des aktuellen befehls.

   \item[ctrl-a] zum Zeilenanfang
 
   \item[ctrl-k] löschen bis zum Ende der Zeile

   \item[ctrl-r] ``\textbf{r}everse incremental search'' - suche in der history durch Eingabe von Buchstaben.

   \item[ctrl-s] stopped die ausgabe am Terminel.

   \item[ctrl-q] ausgabe wieder erlauben. (Continue).

   \item[ctrl-z] stopped den gerade laufenden Befehl. Duch Eingabe von \textbf{bg} wird dieser in Hintergrund geschickt und läuft dort weiter.
Damit ist die shell wieder frei zur Eingabe weiterer Befehle. Will man/frau
einen Befehl gleich im Hintergrund starten geht das mit einem \& am Ende des
Befehls. z.B.: \texttt{sleep 30 \&}. Mit \textbf{fg} kann ein im Hintergrund
laufener Prozess wieder in den Vordergrund geholt werden. \textbf{jobs}
zeigt alle im Hintergrund laufenden jobs der aktuellen shell. (Siehe weiter
unten für das Verwalten aller jobs, nicht nur derer aus der aktuellebn shell).

   \item[ctrl-d] ist das ``Datei-Ende'' und wird auch von vielen Programmen
bentutzt die Eingabe lesen, um diese Eingabe zu Beenden. Mit dem Befehl
\textbf{exit} oder mit einem \textbf{Ctrl-D} kann eine shell geschlossen
werden.
  

  \item[TAB] die Tabulator Taste dient zur Verfollständigung von Eingaben.
Einmal gedrückt macht sie, falls möglich, eine Verfollständigung: Zweimal
gedrückt macht sie Vorschläge. Normalerweise werden Datei- und
Verzeichnisnamen und die Namen eigegebener Programme vervollständigt.
Inzwischen sind aber auch Bash-Plugins gängig die, z.B. die gängigen
Befehlsoptionen von Befehlen kennen und auch Vervollständigen können. Damit
ist die Eingabe von Befehlen sehr komfortabel und schnell.


\end{description}

\subsection{Wer ist eingelogt?}

\textbf{who} und  \textbf{w} zeigen dir wer gerade aller am selben Computer
eingelogt ist. \textbf{last} zeigt dir wann du zuletzt eingeloggt warst.

\subsection{Diverse kleine, nüztliche Tools}

\begin{description}
  \item[cal] zeigt dir einen aktuellen Kalender. \texttt{cal 1 2025} zeigt
dir den Kalender von Jänner 2025.

  \item[gcal] ist ein erweiterter Kalender. \texttt{gcal -n -qAT} zeigt dir
eine Liste aller österreichischen Feiertage im aktuellen Jahr.

   \item[date] - zeigt Datum, Uhrzeit und Zeitzone an.

   \item[factor] - Primfaktoren Zerlegung. Das geht erstaunlicherweise bis
hin zu relativ großen Zahlen: \texttt{factor 38070848173552587069450139085771113793}

   \item[ping] schickt ein ping Packet zum angegeben Server.

   \item[echo] gibt die angegeben Argumente wieder aus. z.B.: \texttt{echo hallo da}

   \item[wc] zählt Zeichen, Wörter und Zeilen in einer Datei

   \item[seq] macht eine Zahlensequenz: z.b: \texttt[seq 1 10] die Zahlen 1 bis 10.

\end{description}


\subsection{Ausgabeumleitung in und von Dateien}

Das großer-Zeichen: $>$ leitet die Ausgabe eines Befehls, die ansonsten am
Bildschrim landen würde (auch STDOUT genannt) in ein File um. 

z.B.:

\begin{verbatim}

  date > heute.txt

\end{verbatim}

Würde die Ausgabe vom \texttt{date} Befehl in die Datei namens ``heute.txt''
schreiben. \textbf{Achtung:} Falls heute.txt schon existiert wird es kommentarlos
überschrieben!

Den Inhalt der neuerstellten Datei können wir uns mit einem Editor oder mit dem \texttt{cat} Befehl
ansehen: z.B.: \texttt{cat heute.txt}

Cat gibt ein oder mehrere Datein am Bildschirm aus. Natürlich lässt sich
auch die Ausgabe von cat wieder mit $>$ umleiten. Das Minuszeichen $-$ wird
sehr oft an Stellen benutzt an denen ein Filename verwendet werdekn kann um
statt dessen die Eingabe (also die Zeichen die wir gerade eintippen, auch
STDIN genann) zu lesen. 

\texttt{cat - > test.txt} liest die Eingaben die wir tippen und schreibt sie
in eine Datei namens text.txt. (Beenden der Eingabe mit \textbf{Ctrl-D}).

Analog dazu macht das Kleiner-Zeichen $<$ eine Umleitung einer Eingabe.
Ein Programm das normalerweise eine Eingabe lesen würde bekommt nun die
daten aus einer Datei. z.B:

\begin{verbatim}

  seq 1 1000000 > mille.txt
  wc < mille.txt

\end{verbatim}
  
Die Zeile mit dem \texttt{seq 1 1000000 > mille.txt} erzeugt eine Datei
namens mille.txt mit den Zahlen $1$ bis $1000 000$  als Inhalt. (Eine Zahl pro
Zeile). Diese Dateil leiten wir indas \texttt{wc} Tool um zu zählen wieviele
Zeilen, Wörter und Zeichen dort drinnen sind. 

Verwendet man statt einem $>$ zwei $>>$ so bedeutet das bei der
Ausgabeumleitung, dass man eine eventuell vorhandene Datei nicht
überschreiben will sondern den Inhalt am Ende \textbf{anfügen} will.

\begin{verbatim}
echo noch eine zeile >> mille.txt
\end{verbatim}

macht unsere mille.txt Datei noch um eine Zeile länger. 

Fehlermeldungen werden von den meisten Tools nicht über die STDOUT Ausgabe
ausgegeben sondern über einen einen Kanal ``gesndet''. Der wird STDERR
genannt und hat im unix die nummer 2. Will man dies Ausgabe auch umleiten
dann Verwendet man \fbox{$2>$}. Soll die STDERR Ausgabe wie eine normale Ausgabe
behandelt werden so können wir das mit dem Kürzel  \fbox{$2>\&1$} machen.

z.B.:

\begin{verbatim}
  
  ls /etc /gibtsnicht  >bla.txt   2>&1 

\end{verbatim}

\subsection{wildcards in der shell}

* und ? haben in der shell eine spezielle Bedeuting. Sie sind ``Wildcards''
  in Datei- und Verzeichnisnamen. Ein Stern * wird durch ein oder mehrere
Beliebige Zeichen ersetzt. Ein Fragezeichen ? wird durch genau ein
beliebiges Zeichen ersetzt:

\begin{description}
  \item[cat *.txt] gibt alle files aus dem aktuellen Verzeichnis, die auf .txt enden aus.

  \item[gimp bild?.jpg] ruft das Programm \textbf{gimp} auf und übergibt
alle Dateien die mit bild beginnen und dann genau ein zeichen haben. Also
bild1.jpg und bildx.jpg aber \textbf{nicht} bild22.jpg.

\end{description}
  
\subsection{Dateien kopieren, verschieben und verlinken}


\begin{description}

  \item[cp] steht für copy. \texttt{cp quelle.txt ziel.txt} würde die Dateie
quelle.txt auf die datei zeil.txt kopieren. (\textbf{Achtung:} falls die
Datei ziel.txt schon existiert wird sie überschrieben). Ist das letzte
Argument ein Verzeichnis so werden die Files (es können in diesem Falle
mehrerer sein) mit dem selben Namen ins Zielverzeichnis kopiert. (Wenn mehr
als 2 Argumente angegeben werden, so muss das letzte Argument ein
Verzeichnis sein). cp hat viele Optionen u.a. die Option -r für
``rekursiv'', falls Verzeichnisse samt aller Unterordner kopiert werden
sollen.

 \item[mv] funktiniert ähnlich wie cp, aber es wird nicht kopiert sondern verschoben (move).

 \item[ln -s] erstellt einen, so genannten ``sym-link''. Das schaut aus wie
eine Datei ist aber nur ein Verweis auf eine Datei oder ein Verzeichnis. Das
ist praktisch um sich z.b. ``abkürzungen'' zu erstellen. z.B.:
\texttt{ln -s /usr/share/sounds Desktop/klaenge} würde einen Link namens
``klaenge'' im Verzeichnis Desktop erstellen das auf den Systemordner zeigt
in dem Linux viele Audiofiles abgelegt hat.


\end{description}


\subsection{Hilfe!}

Neben Doktor google, gibt es auch direkt im Linux viel eingebaute Hilfe. 

Viele Befehle haben direkt Hilfe eingebaut. Mit den Optionen \textbf{-h}
\textbf{--help} oder \textbf{-help} (leider nicht einheitlich) wird
eine kurze Hilfe angezeigt. Diese Hilfe ist meist nur kurz und listet meist
nur die Optionen auf. 

Der \textbf{help} Befehl zeigt Hilfe zur aktuellen shell (BASH) an.
\textbf{help echo} würde z.B. zum (intern in der shell eingebauten) echo
Befehl anzeigen. Das funktioniert aber nur bei Befehlen die direkt, intern
in der shell eingebaut sind. Für die meisten Befehle brauchen wir daher eine
andere Hilfe:

\textbf{man} ist der Befehl zum aufruf der \textbf{man}ual Seiten. Die
meisten Befehle haben solche man-pages. 

\texttt{man echo} würde z.b. die man page zum echo Befehl ausgeben (der
neben dem eingeboten echo kommando ebenfalls existiert).

\textbf{man} verwendet zur Anzeige der Seiten einen so genannten ``Pager''.
Da viele man-pages sehr lang sind kann man/frau damit in den Pages blättern.

Hier die wichtigsten Keyboard-Shortcuts für den standard Pager. Der
standard-Pager im Linux ist ``less'' und ist eine weiterentwicklung von
``more''.

\begin{description}

  \item[Leertaste] zum weiterblättern. 
  \item[q] beenden
  \item[Pfeiltasten oder j k] um jeweils eine Zeile nach unten oder oben zu verschieben.
  \item[/suchstring] sucht nacht ``suchstring''. (Wobei ``suchstring'' eine
  so genannte ``regular expression'' ist - mehr dazu später).
  \item[n] die vorgehende Suche wiederhohlen (weitersuchen  -\textbf{n}ext).
  \item[N] weitersuchen nach oben (also in die andere Richtung).

\end{description}

Viele GNU Tools haben neben einer man-page auch eine ``info'' page.
\textbf{info} ist etwas mühsam zu bediene, aber kann dafür Links zu anderen
info Seiten enthalten. Eine frühe Form von ``hypertext''.


  - bultin vs. external

  - help
 
  - man

  - -h --help

  - ps

  - top


 

* advanced CLI  ( 20 min )

  - ssh user@host erwaehnen

  - find
     o name
     o mtime 
     o type 

  - pipe |

  - more, less

  - grep 

  - sed

  - tr

  - ping

  - host

  - environment varialben

    o set
    o echo $x $?
    o export 
    o $HOME $HOSTNAME $PATH $EDITOR
 

  - command substitution: backticks und \$( )
  - find exec, xargs, xargs -0, xargs -n

%  - ein einfaches shell script mit #!/bin/bash

  - permissions:

      o chmod ugo+rx
      o chown user, chown user.group 

  - gnu  and tools

    o seq
    o bc
    o cut
    o sort, uniq
    o rev


  - for loop

  - if 

%  - bashrc, bash_profile .profile

  - alias

  - builtin vs external

  - which 

  - su -, sudo, id
 
  - passwd

* useful CLI tools  ( 15 min )

  * tar

     c,v,x,p

  * gzip

  * locate

  * pdftk

  * cal gcal -n

  * w3m, curl, wget

  * mutt

  * host, ping, whois

  * minicom

  * factor

  * bc

  * dict, fortune 

  * irc 

  * figlet

  * dd

  * rsync


\end{document}
 